@@ -20,25 +20,20 @@ extern "C"
 void *alloca (size_t);
 #endif
 
-#include <stdio.h>
-#include <string.h>
 #include <ctype.h>
-#include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
 #include <sys/mman.h>
 
 #include <Ecore_File.h>
 
-#include "Efreet.h"
-#include "efreet_private.h"
-
-#ifdef EFREET_MODULE_LOG_DOM
-#undef EFREET_MODULE_LOG_DOM
-#endif
+/* define macros and variable for using the eina logging system  */
 #define EFREET_MODULE_LOG_DOM _efreet_ini_log_dom
 static int _efreet_ini_log_dom = -1;
 
+#include "Efreet.h"
+#include "efreet_private.h"
+
 static Eina_Hash *efreet_ini_parse(const char *file);
 static const char *efreet_ini_unescape(const char *str) EINA_ARG_NONNULL(1);
 static Eina_Bool
@@ -55,10 +50,10 @@ int
 efreet_ini_init(void)
 {
     _efreet_ini_log_dom = eina_log_domain_register
-      ("efreet_init", EFREET_DEFAULT_LOG_COLOR);
+      ("efreet_ini", EFREET_DEFAULT_LOG_COLOR);
     if (_efreet_ini_log_dom < 0)
     {
-        ERROR("Efreet: Could not create a log domain for efreet_init");
+        EINA_LOG_ERR("Efreet: Could not create a log domain for efreet_ini");
         return 0;
     }
     return 1;
@@ -73,15 +68,9 @@ void
 efreet_ini_shutdown(void)
 {
     eina_log_domain_unregister(_efreet_ini_log_dom);
+    _efreet_ini_log_dom = -1;
 }
 
-/**
- * @param file The file to parse
- * @return Returns a new Efreet_Ini structure initialized with the contents
- * of @a file, or NULL on memory allocation failure
- * @brief Creates and initializes a new Ini structure with the contents of
- * @a file, or NULL on failure
- */
 EAPI Efreet_Ini *
 efreet_ini_new(const char *file)
 {
@@ -131,6 +120,9 @@ efreet_ini_parse(const char *file)
     }
 
     left = file_stat.st_size;
+    /* let's make mmap safe and just get 0 pages for IO erro */
+    eina_mmap_safety_enabled_set(EINA_TRUE);
+   
     buffer = mmap(NULL, left, PROT_READ, MAP_SHARED, fileno(f), 0);
     if (buffer == MAP_FAILED)
     {
@@ -197,22 +189,22 @@ efreet_ini_parse(const char *file)
             {
                 /* invalid file - skip line? or refuse to parse file? */
                 /* just printf for now till we figure out what to do */
-//                printf("Invalid file (%s) (missing ] on group name)\n", file);
+//                ERR("Invalid file (%s) (missing ] on group name)", file);
             }
             goto next_line;
         }
 
         if (!section)
         {
-//            INF("Invalid file (%s) (missing section)", file);
-            goto next_line;
+            INF("Invalid file (%s) (missing section)", file);
+            goto error;
         }
 
         /* find for '=' */
         for (sep = 0; (sep < line_length) && (line_start[sep] != '='); ++sep)
             ;
 
-        if (sep < line_length)
+        if (sep < line_length && line_start[sep] == '=')
         {
             char *key, *value;
             int key_end, value_start, value_end;
@@ -265,11 +257,12 @@ efreet_ini_parse(const char *file)
             eina_hash_del_by_key(section, key);
             eina_hash_add(section, key, efreet_ini_unescape(value));
         }
-//        else
-//        {
-//            /* invalid file... */
-//            INF("Invalid file (%s) (missing = from key=value pair)", file);
-//        }
+        else
+        {
+            /* invalid file... */
+            INF("Invalid file (%s) (missing = from key=value pair)", file);
+            goto error;
+        }
 
 next_line:
         left -= line_length + 1;
@@ -286,13 +279,11 @@ next_line:
     }
 #endif
     return data;
+error:
+    if (data) eina_hash_free(data);
+    return NULL;
 }
 
-/**
- * @param ini The Efreet_Ini to work with
- * @return Returns no value
- * @brief Frees the given Efree_Ini structure.
- */
 EAPI void
 efreet_ini_free(Efreet_Ini *ini)
 {
@@ -302,12 +293,6 @@ efreet_ini_free(Efreet_Ini *ini)
     FREE(ini);
 }
 
-/**
- * @param ini The Efreet_Ini to work with
- * @param file The file to load
- * @return Returns no value
- * @brief Saves the given Efree_Ini structure.
- */
 EAPI int
 efreet_ini_save(Efreet_Ini *ini, const char *file)
 {
@@ -330,12 +315,6 @@ efreet_ini_save(Efreet_Ini *ini, const char *file)
     return 1;
 }
 
-/**
- * @param ini The Efreet_Ini to work with
- * @param section The section of the ini file we want to get values from
- * @return Returns 1 if the section exists, otherwise 0
- * @brief Sets the current working section of the ini file to @a section
- */
 EAPI int
 efreet_ini_section_set(Efreet_Ini *ini, const char *section)
 {
@@ -345,12 +324,6 @@ efreet_ini_section_set(Efreet_Ini *ini, const char *section)
     return (ini->section ? 1 : 0);
 }
 
-/**
- * @param ini The Efreet_Ini to work with
- * @param section The section of the ini file we want to add
- * @return Returns no value
- * @brief Adds a new working section of the ini file to @a section
- */
 EAPI void
 efreet_ini_section_add(Efreet_Ini *ini, const char *section)
 {
@@ -366,13 +339,6 @@ efreet_ini_section_add(Efreet_Ini *ini, const char *section)
     eina_hash_add(ini->data, section, hash);
 }
 
-/**
- * @param ini The Efree_Ini to work with
- * @param key The key to lookup
- * @return Returns the string associated with the given key or NULL if not
- * found.
- * @brief Retrieves the value for the given key or NULL if none found.
- */
 EAPI const char *
 efreet_ini_string_get(Efreet_Ini *ini, const char *key)
 {
@@ -381,13 +347,6 @@ efreet_ini_string_get(Efreet_Ini *ini, const char *key)
     return eina_hash_find(ini->section, key);
 }
 
-/**
- * @param ini The Efree_Ini to work with
- * @param key The key to use
- * @param value The value to set
- * @return Returns no value
- * @brief Sets the value for the given key
- */
 EAPI void
 efreet_ini_string_set(Efreet_Ini *ini, const char *key, const char *value)
 {
@@ -397,13 +356,6 @@ efreet_ini_string_set(Efreet_Ini *ini, const char *key, const char *value)
     eina_hash_add(ini->section, key, eina_stringshare_add(value));
 }
 
-/**
- * @param ini The Efree_Ini to work with
- * @param key The key to lookup
- * @return Returns the integer associated with the given key or -1 if not
- * found.
- * @brief Retrieves the value for the given key or -1 if none found.
- */
 EAPI int
 efreet_ini_int_get(Efreet_Ini *ini, const char *key)
 {
@@ -417,13 +369,6 @@ efreet_ini_int_get(Efreet_Ini *ini, const char *key)
     return -1;
 }
 
-/**
- * @param ini The Efree_Ini to work with
- * @param key The key to use
- * @param value The value to set
- * @return Returns no value
- * @brief Sets the value for the given key
- */
 EAPI void
 efreet_ini_int_set(Efreet_Ini *ini, const char *key, int value)
 {
@@ -435,13 +380,6 @@ efreet_ini_int_set(Efreet_Ini *ini, const char *key, int value)
     efreet_ini_string_set(ini, key, str);
 }
 
-/**
- * @param ini The Efree_Ini to work with
- * @param key The key to lookup
- * @return Returns the double associated with the given key or -1 if not
- * found.
- * @brief Retrieves the value for the given key or -1 if none found.
- */
 EAPI double
 efreet_ini_double_get(Efreet_Ini *ini, const char *key)
 {
@@ -455,13 +393,6 @@ efreet_ini_double_get(Efreet_Ini *ini, const char *key)
     return -1;
 }
 
-/**
- * @param ini The Efree_Ini to work with
- * @param key The key to use
- * @param value The value to set
- * @return Returns no value
- * @brief Sets the value for the given key
- */
 EAPI void
 efreet_ini_double_set(Efreet_Ini *ini, const char *key, double value)
 {
@@ -477,12 +408,6 @@ efreet_ini_double_set(Efreet_Ini *ini, const char *key, double value)
     efreet_ini_string_set(ini, key, str);
 }
 
-/**
- * @param ini The ini struct to work with
- * @param key The key to search for
- * @return Returns 1 if the boolean is true, 0 otherwise
- * @brief Retrieves the boolean value at key @a key from the ini @a ini
- */
 EAPI unsigned int
 efreet_ini_boolean_get(Efreet_Ini *ini, const char *key)
 {
@@ -496,13 +421,6 @@ efreet_ini_boolean_get(Efreet_Ini *ini, const char *key)
     return 0;
 }
 
-/**
- * @param ini The ini struct to work with
- * @param key The key to use
- * @param value The value to set
- * @return Returns no value
- * @brief Sets the value for the given key
- */
 EAPI void
 efreet_ini_boolean_set(Efreet_Ini *ini, const char *key, unsigned int value)
 {
@@ -512,13 +430,6 @@ efreet_ini_boolean_set(Efreet_Ini *ini, const char *key, unsigned int value)
     else efreet_ini_string_set(ini, key, "false");
 }
 
-/**
- * @param ini The ini struct to work with
- * @param key The key to search for
- * @return Returns the utf8 encoded string associated with @a key, or NULL
- *         if none found
- * @brief Retrieves the utf8 encoded string associated with @a key in the current locale or NULL if none found
- */
 EAPI const char *
 efreet_ini_localestring_get(Efreet_Ini *ini, const char *key)
 {
@@ -575,13 +486,6 @@ efreet_ini_localestring_get(Efreet_Ini *ini, const char *key)
     return val;
 }
 
-/**
- * @param ini The ini struct to work with
- * @param key The key to use
- * @param value The value to set
- * @return Returns no value
- * @brief Sets the value for the given key
- */
 EAPI void
 efreet_ini_localestring_set(Efreet_Ini *ini, const char *key, const char *value)
 {
@@ -616,12 +520,6 @@ efreet_ini_localestring_set(Efreet_Ini *ini, const char *key, const char *value)
     efreet_ini_string_set(ini, buf, value);
 }
 
-/**
- * @param ini The ini struct to work with
- * @param key The key to remove
- * @return Returns no value
- * @brief Remove the given key from the ini struct
- */
 EAPI void
 efreet_ini_key_unset(Efreet_Ini *ini, const char *key)
 {
