@@ -3,7 +3,6 @@
 #include "config.h"
 #include <stdio.h>
 #include <string.h>
-#include <sys/types.h>
 #include <sys/time.h>
 #include <Ecore.h>
 
@@ -20,10 +19,10 @@ ef_mime_cb_get(void)
         char *file;
         char *mime;
     } files[] = {
-        {PACKAGE_DATA_DIR"/test/test_type.desktop", "application/x-desktop"},
-        {PACKAGE_DATA_DIR"/test/entry.png", "image/png"},
-        {PACKAGE_DATA_DIR"/test/entry", "image/png"},
-        {PACKAGE_DATA_DIR"/test/sub", "inode/directory"},
+        {PKG_DATA_DIR"/test/test_type.desktop", "application/x-desktop"},
+        {PKG_DATA_DIR"/test/entry.png", "image/png"},
+        {PKG_DATA_DIR"/test/entry", "image/png"},
+        {PKG_DATA_DIR"/test/sub", "inode/directory"},
         {NULL, NULL}
     };
     double start;
